#include "StdAfx.h"
#include "Device.h"

CDevice::CDevice(void)
{
}

CDevice::CDevice(bool bLog, unsigned nPort) 
: CSerial(bLog, nPort, 38400), m_nStation(-1), m_nSlot(-1), m_nArm(1), m_nInitialStatus(-1),  m_nFGReadyStatus(-1),
 m_nPutIntoStatus(-1), m_nGetFromStatus(-1), m_nMapScanStatus(-1), m_nVacCheckStatus(-1)
{
	m_hStopEvent = CreateEvent(NULL, TRUE, FALSE, NULL);

	assert(m_hStopEvent);

	SetTimeout(1);
	BeginThread();
}

CDevice::~CDevice(void)
{
	SetEvent(m_hStopEvent);
	WaitFor();

	CloseHandle(m_hStopEvent);
}

void CDevice::OnExecute()
{
	while( WaitForSingleObject(m_hStopEvent, 1) )
	{
		CIronLock cLock(*this);

		if (InWaiting() > 0)
		{
			string strRead = ReadLine('\n');

			CheckComplete(strRead);
		}
	}
}

bool CDevice::CheckComplete(string strRead)
{
	if (strRead.find(">FGREADY") < strRead.length() && m_nFGReadyStatus == R_MOVING)
	{
		m_nFGReadyStatus = R_COMPLETE;
	}
	else if (strRead.find(">PUTINTO") < strRead.length() && m_nPutIntoStatus == R_MOVING)
	{
		m_nPutIntoStatus = R_COMPLETE;
	}
	else if (strRead.find(">GETFROM") < strRead.length() && m_nGetFromStatus == R_MOVING)
	{
		m_nGetFromStatus = R_COMPLETE;
	}
	else if (strRead.find(">MAPSCAN") < strRead.length() && m_nMapScanStatus == R_MOVING)
	{
		m_nMapScanStatus = R_COMPLETE;
	}
	else if (strRead.find(">INITIAL") < strRead.length() && m_nInitialStatus == R_MOVING)
	{
		m_nInitialStatus = R_COMPLETE;
	}
	else if (strRead.find(">VACCHEK") < strRead.length() && m_nVacCheckStatus == R_MOVING)
	{
		m_nVacCheckStatus = R_COMPLETE;
	}
	else
	{
		return false;
	}

	return true;
}

bool CDevice::CheckPosition()
{
	
	if (m_nStation < 0 || m_nStation > 11)
	{
		CIronLib::WriteDebugLog("Robostart>>Invalid Station Number[%d].(%s:%d)", m_nStation, __FILE__, __LINE__);
		return false;
	}
	
	if (m_nSlot < 0 || m_nSlot > 26)
	{
		CIronLib::WriteDebugLog("Robostart>>Invalid Slot Number[%d].(%s:%d)", m_nSlot, __FILE__, __LINE__);
		return false;
	}
	
	if (m_nArm < 1 || m_nArm > 2)
	{
		CIronLib::WriteDebugLog("Robostart>>Invalid Arm Number[%d].(%s:%d)", m_nArm, __FILE__, __LINE__);
		return false;
	}

	return true;
}

LONG CDevice::GetLowArmWaferExist(int& nVal)
{
	nVal = m_nLowArmWaferExist;
	return S_OK;
}

LONG CDevice::GetUpperArmWaferExist(int& nVal)
{
	nVal = m_nUpperArmWaferExist;
	return S_OK;
}

LONG CDevice::GetInitialStatus(int& nVal)
{
	nVal = m_nInitialStatus;
	return S_OK;
}

LONG CDevice::GetFGReadyStatus(int& nVal)
{
	nVal = m_nFGReadyStatus;
	return S_OK;
}

LONG CDevice::GetPutIntoStatus(int& nVal)
{
	nVal = m_nPutIntoStatus;
	return S_OK;
}

LONG CDevice::GetGetFromStatus(int& nVal)
{
	nVal = m_nGetFromStatus;
	return S_OK;
}

LONG CDevice::GetMapScanStatus(int& nVal)
{
	nVal = m_nMapScanStatus;
	return S_OK;
}

bool CDevice::GetRobotState()
{
	CIronLock cLock(*this);

	Write("R_STATE\r\n");

	string strRead = ReadLine('\n');

	while (CheckComplete(strRead))
	{
		strRead = ReadLine('\n');
	}

	if (strRead != "-R_STATE\r\n")
	{
		CIronLib::WriteDebugLog("Robostar>>Robot State Error[%s]", strRead.c_str());
		return false;
	}

	strRead = ReadLine('\n');

	while (CheckComplete(strRead))
	{
		strRead = ReadLine('\n');
	}

	if (strRead.length() < 12)
	{
		CIronLib::WriteDebugLog("Robostar>>Robot State Error[%s]", strRead.c_str());
		return false;
	}

	if (strRead.substr(0, 1) == "*")
	{
		m_nRunning = atoi(strRead.substr(1, 1).c_str());
		m_nError = atoi(strRead.substr(3, 1).c_str());
		//m_nServo = atoi(strRead.substr(3, 1).c_str());
		//m_nMode = atoi(strRead.substr(4, 1).c_str());
		//m_nPosition = atoi(strRead.substr(6, 1).c_str());
		//m_nArmState = atoi(strRead.substr(7, 1).c_str());

		//int nSize = atoi(strRead.substr(9,2).c_str());
		m_strMapData = strRead.substr(6, 25);
	}
	else
	{
		FlushInput();
		CIronLib::WriteDebugLog("Robostar>>Robot State Error[%s]", strRead.c_str());
		return false;
	}

	strRead = ReadLine('\n');

	while (CheckComplete(strRead))
	{
		strRead = ReadLine('\n');
	}

	if (strRead != ">R_STATE\r\n")
	{
		CIronLib::WriteDebugLog("Robostar>>Robot State Error[%s]", strRead.c_str());
		return false;
	}

	return true;
}

LONG CDevice::GetRunning(int& nVal)
{
	if (!GetRobotState())
		return E_FAIL;

	nVal = m_nRunning;
	return S_OK;
}

LONG CDevice::GetError(int& nVal)
{
	if (!GetRobotState())
		return E_FAIL;

	nVal = m_nError;
	return S_OK;
}

LONG CDevice::GetServo(int& nVal)
{
	if (!GetRobotState())
		return E_FAIL;

	nVal = m_nServo;
	return S_OK;
}

LONG CDevice::GetMode(int& nVal)
{
	if (!GetRobotState())
		return E_FAIL;

	nVal = m_nMode;
	return S_OK;
}

LONG CDevice::GetPosition(int& nVal)
{
	if (!GetRobotState())
		return E_FAIL;

	nVal = m_nPosition;
	return S_OK;
}

LONG CDevice::GetArmState(int& nVal)
{
	if (!GetRobotState())
		return E_FAIL;

	nVal = m_nArm;
	return S_OK;
}

LONG CDevice::GetMapData(string& strVal)
{
	if (!GetRobotState())
		return E_FAIL;

	strVal = m_strMapData;
	return S_OK;
}

LONG CDevice::SetInit(int nVal)
{
	CIronLock cLock(*this);

	if(nVal == 1)
	{
		m_nInitialStatus = R_MOVING;

		string strWrite = "INITIAL\r\n";
		string strMsg = "INITIAL\r\n";

		if (!Write(strWrite))
			return E_FAIL;

		string strRead = ReadLine('\n');

		while (CheckComplete(strRead))
		{
			strRead = ReadLine('\n');
		}

		if (strRead != "-" + strMsg)
		{
			m_nInitialStatus = R_ERROR;
			CIronLib::WriteDebugLog("Robostar>>Initial Error[%s]", strMsg.c_str());
			return E_FAIL;
		}
	}

	return S_OK;
}

LONG CDevice::SetServo(int nVal)
{
	CIronLock cLock(*this);

	if (nVal == 0)
	{
		if (!Write("SERVOOFF\r\n"))
			return E_FAIL;

		string strRead = ReadLine('\n');

		while (CheckComplete(strRead))
		{
			strRead = ReadLine('\n');
		}

		if (strRead != "-SERVOOFF\r\n")
		{
			CIronLib::WriteDebugLog("Robostar>>Servo Off Error[%s]", strRead.c_str());
			return E_FAIL;
		}

		strRead = ReadLine('\n');

		while (CheckComplete(strRead))
		{
			strRead = ReadLine('\n');
		}

		if (strRead != ">SERVOOFF\r\n")
		{
			CIronLib::WriteDebugLog("Robostar>>Servo Off Timeout");
			return E_FAIL;
		}
	}
	else if(nVal == 1)
	{
		if (!Write("SERVOON\r\n"))
			return E_FAIL;

		string strRead = ReadLine('\n');

		while (CheckComplete(strRead))
		{
			strRead = ReadLine('\n');
		}

		if (strRead != "-SERVOON\r\n")
		{
			CIronLib::WriteDebugLog("Robostar>>Servo On Error[%s]", strRead.c_str());
			return E_FAIL;
		}

		strRead = ReadLine('\n');

		while (CheckComplete(strRead))
		{
			strRead = ReadLine('\n');
		}

		if (strRead != ">SERVOON\r\n")
		{
			CIronLib::WriteDebugLog("Robostar>>Servo On Timeout");
			return E_FAIL;
		}
	}

	return S_OK;
}

LONG CDevice::SetStation(int nVal)
{
	if (nVal < 0 || nVal > 11)
	{
		CIronLib::WriteDebugLog("Robostart>>Invalid Station Number[%d].(%s:%d)", nVal, __FILE__, __LINE__);
		return E_FAIL;
	}

	m_nStation = nVal;

	return S_OK;
}

LONG CDevice::SetSlot(int nVal)
{
	if (nVal < 0 || nVal > 26)
	{
		CIronLib::WriteDebugLog("Robostart>>Invalid Slot Number[%d].(%s:%d)", nVal, __FILE__, __LINE__);
		return E_FAIL;
	}

	m_nSlot = nVal;

	return S_OK;
}

LONG CDevice::SetArm(int nVal)
{
	if (nVal < 1 || nVal > 2)
	{
		CIronLib::WriteDebugLog("Robostart>>Invalid Slot Number[%d].(%s:%d)", nVal, __FILE__, __LINE__);
		return E_FAIL;
	}

	m_nArm = nVal;

	return S_OK;
}

LONG CDevice::SetFGReady(int nVal)
{
	CIronLock cLock(*this);

	if(nVal == 1)
	{
		m_nFGReadyStatus = R_MOVING;

		if (!CheckPosition())
			return E_FAIL;

		string strWrite = "FGREADY ";
		string strMsg = "FGREADY\r\n";

		strWrite += CIronLib::itostring(m_nStation) + ",";
		strWrite += CIronLib::itostring(m_nSlot) + ",";
		strWrite += CIronLib::itostring(m_nArm) + "\r\n";

		if (!Write(strWrite))
			return E_FAIL;

		string strRead = ReadLine('\n');

		while (CheckComplete(strRead))
		{
			strRead = ReadLine('\n');
		}

		if (strRead != "-" + strMsg)
		{
			m_nFGReadyStatus = R_ERROR;
			CIronLib::WriteDebugLog("Robostar>>FGReady Error[%s]", strMsg.c_str());
			return E_FAIL;
		}
	}

	return S_OK;
}

LONG CDevice::SetPutInto(int nVal)
{
	CIronLock cLock(*this);

	if(nVal == 1)
	{
		m_nPutIntoStatus = R_MOVING;

		if (!CheckPosition())
			return E_FAIL;

		string strWrite = "PUTINTO ";
		string strMsg = "PUTINTO\r\n";

		strWrite += CIronLib::itostring(m_nStation) + ",";
		strWrite += CIronLib::itostring(m_nSlot) + ",";
		strWrite += CIronLib::itostring(m_nArm) + "\r\n";

		if (!Write(strWrite))
			return E_FAIL;

		string strRead = ReadLine('\n');

		while (CheckComplete(strRead))
		{
			strRead = ReadLine('\n');
		}

		if (strRead != "-" + strMsg)
		{
			m_nPutIntoStatus = R_ERROR;
			CIronLib::WriteDebugLog("Robostar>>PutInto Error[%s]", strMsg.c_str());
			return E_FAIL;
		}
	}

	return S_OK;
}

LONG CDevice::SetGetFrom(int nVal)
{
	CIronLock cLock(*this);

	if(nVal == 1)
	{
		m_nGetFromStatus = R_MOVING;

		if (!CheckPosition())
			return E_FAIL;

		string strWrite = "GETFROM ";
		string strMsg = "GETFROM\r\n";

		strWrite += CIronLib::itostring(m_nStation) + ",";
		strWrite += CIronLib::itostring(m_nSlot) + ",";
		strWrite += CIronLib::itostring(m_nArm) + "\r\n";

		if (!Write(strWrite))
			return E_FAIL;

		string strRead = ReadLine('\n');

		while (CheckComplete(strRead))
		{
			strRead = ReadLine('\n');
		}

		if (strRead != "-" + strMsg)
		{
			m_nGetFromStatus = R_ERROR;
			CIronLib::WriteDebugLog("Robostar>>GetFrom Error[%s]", strMsg.c_str());
			return E_FAIL;
		}
	}

	return S_OK;
}

LONG CDevice::SetMapScan(int nVal)
{
	CIronLock cLock(*this);

	if(nVal == 1)
	{
		m_nMapScanStatus = R_MOVING;

		if (!CheckPosition())
			return E_FAIL;

		string strWrite = "MAPSCAN ";
		string strMsg = "MAPSCAN\r\n";

		strWrite += CIronLib::itostring(m_nStation) + ",25\r\n";

		if (!Write(strWrite))
			return E_FAIL;

		string strRead = ReadLine('\n');

		while (CheckComplete(strRead))
		{
			strRead = ReadLine('\n');
		}

		if (strRead != "-" + strMsg)
		{
			m_nMapScanStatus = R_ERROR;
			CIronLib::WriteDebugLog("Robostar>>MapScan Error[%s]", strMsg.c_str());
			return E_FAIL;
		}
	}
	return S_OK;
}

LONG CDevice::SetStop(int nVal)
{
	CIronLock cLock(*this);

	if(nVal == 1)
	{
		if (!Write("R_RSTOP\r\n"))
			return E_FAIL;

		string strRead = ReadLine('\n');

		while (CheckComplete(strRead))
		{
			strRead = ReadLine('\n');
		}

		if (strRead != "-R_RSTOP\r\n")
		{
			CIronLib::WriteDebugLog("Robostar>>Robot Stop Error[%s]", strRead.c_str());
			return E_FAIL;
		}

		strRead = ReadLine('\n');

		while (CheckComplete(strRead))
		{
			strRead = ReadLine('\n');
		}

		if (strRead != ">R_RSTOP\r\n")
		{
			CIronLib::WriteDebugLog("Robostar>>Robot Stop Timeout");
			return E_FAIL;
		}
	}

	return S_OK;
}

LONG CDevice::SetResume(int nVal)
{
	CIronLock cLock(*this);

	if(nVal == 1)
	{
		if (!Write("R_RESUME\r\n"))
			return E_FAIL;

		string strRead = ReadLine('\n');

		while (CheckComplete(strRead))
		{
			strRead = ReadLine('\n');
		}

		if (strRead != "-R_RESUME\r\n")
		{
			CIronLib::WriteDebugLog("Robostar>>Robot Resume Error[%s]", strRead.c_str());
			return E_FAIL;
		}

		strRead = ReadLine('\n');

		while (CheckComplete(strRead))
		{
			strRead = ReadLine('\n');
		}

		if (strRead != ">R_RESUME\r\n")
		{
			CIronLib::WriteDebugLog("Robostar>>Robot Resume Timeout");
			return E_FAIL;
		}
	}

	return S_OK;
}

LONG CDevice::SetReset(int nVal)
{
	CIronLock cLock(*this);

	if(nVal == 1)
	{
		if (!Write("R_RESET\r\n"))
			return E_FAIL;

		string strRead = ReadLine('\n');

		while (CheckComplete(strRead))
		{
			strRead = ReadLine('\n');
		}

		if (strRead != "-R_RESET\r\n")
		{
			CIronLib::WriteDebugLog("Robostar>>Robot Resume Error[%s]", strRead.c_str());
			return E_FAIL;
		}

		strRead = ReadLine('\n');

		while (CheckComplete(strRead))
		{
			strRead = ReadLine('\n');
		}

		if (strRead != ">R_RESET\r\n")
		{
			CIronLib::WriteDebugLog("Robostar>>Robot Resume Timeout");
			return E_FAIL;
		}
	}

	return S_OK;
}

LONG CDevice::SetPause(int nVal)
{
	CIronLock cLock(*this);

	if(nVal == 1)
	{
		if (!Write("R_PAUSE\r\n"))
			return E_FAIL;

		string strRead = ReadLine('\n');

		while (CheckComplete(strRead))
		{
			strRead = ReadLine('\n');
		}

		if (strRead != "-R_PAUSE\r\n")
		{
			CIronLib::WriteDebugLog("Robostar>>Robot Resume Error[%s]", strRead.c_str());
			return E_FAIL;
		}

		strRead = ReadLine('\n');

		while (CheckComplete(strRead))
		{
			strRead = ReadLine('\n');
		}

		if (strRead != ">R_PAUSE\r\n")
		{
			CIronLib::WriteDebugLog("Robostar>>Robot Resume Timeout");
			return E_FAIL;
		}
	}

	return S_OK;
}

LONG CDevice::SetVacCtrl(int nVal)
{
	CIronLock cLock(*this);

	string strWrite = "VACCTRL ";

	strWrite += CIronLib::itostring(m_nArm) + ",";

	if (nVal == 0)
	{
		strWrite += "0\r\n";

		if (!Write(strWrite))
			return E_FAIL;

		string strRead = ReadLine('\n');

		while (CheckComplete(strRead))
		{
			strRead = ReadLine('\n');
		}

		if (strRead != "-" + strWrite)
		{
			CIronLib::WriteDebugLog("Robostar>>Vacuum Control Off Error[%s]", strRead.c_str());
			return E_FAIL;
		}

		strRead = ReadLine('\n');

		while (CheckComplete(strRead))
		{
			strRead = ReadLine('\n');
		}

		if (strRead != ">" + strWrite)
		{
			CIronLib::WriteDebugLog("Robostar>>Vacuum Control Off Timeout");
			return E_FAIL;
		}
	}
	else
	{
		strWrite += "1\r\n";

		if (!Write(strWrite))
			return E_FAIL;

		string strRead = ReadLine('\n');

		while (CheckComplete(strRead))
		{
			strRead = ReadLine('\n');
		}

		if (strRead != "-" + strWrite)
		{
			CIronLib::WriteDebugLog("Robostar>>Vacuum Control On Error[%s]", strRead.c_str());
			return E_FAIL;
		}

		strRead = ReadLine('\n');

		while (CheckComplete(strRead))
		{
			strRead = ReadLine('\n');
		}

		if (strRead != ">" + strWrite)
		{
			CIronLib::WriteDebugLog("Robostar>>Vacuum Control On Timeout");
			return E_FAIL;
		}
	}

	return S_OK;
}

LONG CDevice::SetWaferCheck(int nVal)
{
	CIronLock cLock(*this);

	if(nVal == 1)
	{
		m_nVacCheckStatus = R_MOVING;

		if (!CheckPosition())
			return E_FAIL;

		string strWrite = "VACCHEK ";
		string strMsg = "VACCHEK\r\n";

		strWrite += CIronLib::itostring(m_nArm) + "\r\n";

		if (!Write(strWrite))
			return E_FAIL;

		string strRead = ReadLine('\n');

		while (CheckComplete(strRead))
		{
			strRead = ReadLine('\n');
		}

		if (strRead != "-" + strMsg)
		{
			CIronLib::WriteDebugLog("Robostar>>Vacuum Check Error[%s]", strMsg.c_str());
			return E_FAIL;
		}

		strRead = ReadLine('\n');

		while (CheckComplete(strRead))
		{
			strRead = ReadLine('\n');
		}

		if (strRead.length() < 5 || strRead.substr(0, 1) != "*" || strRead.substr(strRead.length() - 2, 2) != "\r\n") 
		{
			CIronLib::WriteDebugLog("Robostar>>Vacuum Check Error[%s]", strMsg.c_str());
			return E_FAIL;
		}

		if (m_nArm == 1)
		{
			m_nLowArmWaferExist = atoi(strRead.substr(2, 1).c_str());
		}
		else if (m_nArm == 2)
		{
			m_nUpperArmWaferExist = atoi(strRead.substr(2, 1).c_str());
		}
	}

	return S_OK;
}
